-----------------------------------------------------------------------------------------------
-- Client Lua Script for Kinetica
-- Original creation "Volatility" by Aukmoo, repurposed by Aorin
-----------------------------------------------------------------------------------------------

require "Window"

local Kinetica = {}

local userSettings = {

	--General settings
	"setColorChange",
	"setHideWhenEmpty",
		
	--Color settings
	"setAboveColor",
	"setBelowColor",
	
	--Font settings
	"setFont",
	
	--Style settings
	"setWidth",
	"setHeight",
}


-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Kinetica:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function Kinetica:Init()
	Apollo.RegisterAddon(self, false)
end


-----------------------------------------------------------------------------------------------
-- Kinetica OnLoad
-----------------------------------------------------------------------------------------------
function Kinetica:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("Kinetica.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
end

function Kinetica:OnDocumentReady()
	if self.xmlDoc == nil then
		return
	end
	
	self:OnRequiredFlagsChanged()
end

function Kinetica:OnRequiredFlagsChanged()
	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end

function Kinetica:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Warrior then
		return
	end

	--Register slash commands
	Apollo.RegisterSlashCommand("kinetica", 				"OnKineticaOn", self)
	Apollo.RegisterSlashCommand("kin", 						"OnKineticaOn", self)
	
	--Register event handlers
	Apollo.RegisterEventHandler("VarChange_FrameCount",		"OnFrame", self)
	
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "SettingsForm", nil, self)
	self.wndKinetica = Apollo.LoadForm(self.xmlDoc, "KineticaFrame", nil, self)
	self.wndMain:Show(false)
	self.wndKinetica:Show(false)
	
	self.wndMain:FindChild("Button_Options"):SetCheck(true)
	self.wndMain:FindChild("Content:About"):Show(false)
	
	self:DefaultSettings()
	self:RefreshSettings()
	self:OptionsChanged()
end

function Kinetica:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	local tSavedData = {}
	
	tSavedData.setAnchorOffsets = { self.wndKinetica:GetAnchorOffsets() }
	for _,v in ipairs(userSettings) do tSavedData[v] = self[v] end
	
	return tSavedData
end

function Kinetica:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	if tSavedData.setAnchorOffsets then
		self.setAnchorOffsets = tSavedData.setAnchorOffsets
	end
	
	for _,v in ipairs(userSettings) do
		if tSavedData[v] ~= nil then self[v] = tSavedData[v] end
	end
end

function Kinetica:OnKineticaOn()
	self.wndMain:Show(true)
	self:RefreshSettings()
end

function Kinetica:OnConfigure()
	if self.wndMain ~= nil then
		self:OnKineticaOn()
	end
end

function Kinetica:DefaultSettings()
	
	--General settings
	if self.setAnchorOffsets == nil then self.setAnchorOffsets = { 0, 0, 360, 30 } end
	if self.setColorChange == nil then self.setColorChange = true end
		if self.setHideWhenEmpty == nil then self.setHideWhenEmpty = false end
		
	
	--Color settings
	if self.setOpacity == nil then self.setOpacity = "FF" end
	if self.setAboveColor == nil then self.setAboveColor = "FFDB00" end
	if self.setBelowColor == nil then self.setBelowColor = "9D1721" end
	
	--Font settings
	if self.setFont == nil then self.setFont = "Nameplates" end
	
	--Style settings
	if self.setWidth == nil then self.setWidth = 360 end
	if self.setHeight == nil then self.setHeight = 30 end
end

function Kinetica:RefreshSettings()

	--Toggles
	if self.setColorChange ~= nil then
		self.wndMain:FindChild("Button_ColorChange"):SetCheck(self.setColorChange) end
		if self.setHideWhenEmpty ~= nil then
		self.wndMain:FindChild("Button_HideWhenEmpty"):SetCheck(self.setHideWhenEmpty) end
	
	--Colors
	if self.setAboveColor ~= nil then
		self.wndMain:FindChild("Edit_Above"):SetText(self.setAboveColor)
		self.wndMain:FindChild("Edit_Above"):SetTextColor(ApolloColor.new(self.setOpacity..self.setAboveColor)) end
	if self.setBelowColor ~= nil then
		self.wndMain:FindChild("Edit_Below"):SetText(self.setBelowColor)
		self.wndMain:FindChild("Edit_Below"):SetTextColor(ApolloColor.new(self.setOpacity..self.setBelowColor)) end
		
	--Sizes
	if self.setAnchorOffsets ~= nil then
		self.wndMain:FindChild("Edit_Width"):SetText(self.setAnchorOffsets[3] - self.setAnchorOffsets[1])
		self.wndMain:FindChild("Edit_Height"):SetText(self.setAnchorOffsets[4] - self.setAnchorOffsets[2]) end
		
	--Font
	if self.setFont ~= nil then
		self.wndMain:FindChild("Button_SetFontDefault"):SetCheck(self.setFont == "Nameplates")
		self.wndMain:FindChild("Button_SetFontInterface_Small"):SetCheck(self.setFont == "CRB_InterfaceSmall_O")
		self.wndMain:FindChild("Button_SetFontInterface_Medium"):SetCheck(self.setFont == "CRB_InterfaceMedium_O")
		self.wndMain:FindChild("Button_SetFontInterface_Large"):SetCheck(self.setFont == "CRB_InterfaceLarge_O")
		self.wndMain:FindChild("Button_SetFontPixel"):SetCheck(self.setFont == "CRB_Pixel_O")
		self.wndMain:FindChild("Button_SetFontSubtitle"):SetCheck(self.setFont == "Subtitle")
		self.wndMain:FindChild("Button_SetFontThick"):SetCheck(self.setFont == "Thick")
		self.wndMain:FindChild("Button_SetFontFloat"):SetCheck(self.setFont == "CRB_FloaterMedium") end
		
	--[[Style
	if self.setWidth ~= nil then
		self.wndMain:FindChild("Label_WidthDisplay"):SetText(string.format("%s", math.floor(self.setWidth)))
		self.wndMain:FindChild("Slider_Width"):SetValue(self.setWidth) end]]
		
end

function Kinetica:OptionsChanged()
	if setOffsets == nil then
		setOffsets = true
		
		self.wndKinetica:SetAnchorOffsets(unpack(self.setAnchorOffsets))
		self.setAnchorOffsets = { self.wndKinetica:GetAnchorOffsets() }
	end
	
	self.wndKinetica:FindChild("ProgressText"):SetFont(self.setFont)
	self.wndKinetica:Show(true)
end

function Kinetica:OnFrame()
	local unitPlayer = GameLib.GetPlayerUnit()
	local nResourceCurrent = unitPlayer:GetResource(1)
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideWhenEmpty and nResourceCurrent == 0 then
		if self.wndKinetica:IsShown() then
			self.wndKinetica:Show(false)
		end
		return
	end
				
	self.wndKinetica:FindChild("ProgressBar"):SetMax(nResourceMax)
	self.wndKinetica:FindChild("ProgressBar"):SetProgress(nResourceCurrent)
	self.wndKinetica:FindChild("ProgressBar"):SetBarColor("BF"..self.setAboveColor)
	self.wndKinetica:FindChild("ProgressText"):SetText(nResourceCurrent)
	
	if self.setColorChange then
		if (self.setIncludeRegen and nResourceCurrent >= 0 and nResourceCurrent <= 750) or (not self.setIncludeRegen and nResourceCurrent >= 751 and nResourceCurrent <= 1000) then
			self.wndKinetica:FindChild("ProgressBar"):SetBarColor("BF"..self.setBelowColor)
		end
	end
	
	if not self.wndKinetica:IsShown() then
		self.wndKinetica:Show(true)
	end
end


-----------------------------------------------------------------------------------------------
-- Buttons
-----------------------------------------------------------------------------------------------
function Kinetica:Button_Options(wndHandler, wndCtrl)
	self.wndMain:FindChild("Content:General"):Show(true)
	self.wndMain:FindChild("Content:About"):Show(false)
end

function Kinetica:Button_About(wndHandler, wndCtrl)
	self.wndMain:FindChild("Content:General"):Show(false)
	self.wndMain:FindChild("Content:About"):Show(true)
end

function Kinetica:Button_OK()
	self.wndMain:Show(false)
end

function Kinetica:Button_ColorChange(wndHandler, wndCtrl)
	self.setColorChange = wndCtrl:IsChecked()
end

function Kinetica:Button_IncludeRegen(wndHandler, wndCtrl)
	self.setIncludeRegen = wndCtrl:IsChecked()
end

function Kinetica:Button_HideWhenEmpty(wndHandler, wndCtrl)
	self.setHideWhenEmpty = wndCtrl:IsChecked()
end

function Kinetica:Button_ColorReset()
	self:DefaultColors()
	self:RefreshSettings()
end


-----------------------------------------------------------------------------------------------
-- Colors
-----------------------------------------------------------------------------------------------
function Kinetica:Edit_Above()
	local colorString = self.wndMain:FindChild("Edit_Above"):GetText()
	if string.len(colorString) > 6 then self.wndMain:FindChild("Edit_Above"):SetText(string.sub(colorString, 0, 6)) end
	if string.len(colorString) == 6 then self.wndMain:FindChild("Edit_Above"):SetTextColor(ApolloColor.new(self.setOpacity..colorString)) end
	self.setAboveColor = colorString
end

function Kinetica:Edit_Below()
	local colorString = self.wndMain:FindChild("Edit_Below"):GetText()
	if string.len(colorString) > 6 then self.wndMain:FindChild("Edit_Below"):SetText(string.sub(colorString, 0, 6)) end
	if string.len(colorString) == 6 then self.wndMain:FindChild("Edit_Below"):SetTextColor(ApolloColor.new(self.setOpacity..colorString)) end
	self.setBelowColor = colorString
end

-----------------------------------------------------------------------------------------------
-- Sizes
-----------------------------------------------------------------------------------------------
function Kinetica:Edit_Width()
	local newWidth = self.wndMain:FindChild("Edit_Width"):GetText()
	
	if string.len(newWidth) > 2 then
		local currentOffsets = { self.wndKinetica:GetAnchorOffsets() }
		local newOffset = currentOffsets[1] + newWidth
		
		self.setAnchorOffsets = { currentOffsets[1], currentOffsets[2], newOffset, currentOffsets[4] }
		self.wndKinetica:SetAnchorOffsets(unpack(self.setAnchorOffsets))
	end
end

function Kinetica:Edit_Height()
	local newHeight = self.wndMain:FindChild("Edit_Height"):GetText()
	
	if string.len(newHeight) > 1 then
		local currentOffsets = { self.wndKinetica:GetAnchorOffsets() }
		local newOffset = currentOffsets[2] + newHeight
	
		self.setAnchorOffsets = { currentOffsets[1], currentOffsets[2], currentOffsets[3], newOffset }
		self.wndKinetica:SetAnchorOffsets(unpack(self.setAnchorOffsets))
	end
end


-----------------------------------------------------------------------------------------------
-- Fonts
-----------------------------------------------------------------------------------------------
function Kinetica:Button_SetFontDefault()
	self.setFont = "Nameplates"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontInterface_Small()
	self.setFont = "CRB_InterfaceSmall_O"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontInterface_Medium()
	self.setFont = "CRB_InterfaceMedium_O"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontInterface_Large()
	self.setFont = "CRB_InterfaceLarge_O"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontPixel()
	self.setFont = "CRB_Pixel_O"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontSubtitle()
	self.setFont = "Subtitle"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontThick()
	self.setFont = "Thick"
	self:OptionsChanged()
end

function Kinetica:Button_SetFontFloat()
	self.setFont = "CRB_FloaterMedium"
	self:OptionsChanged()
end


-----------------------------------------------------------------------------------------------
-- Sliders
-----------------------------------------------------------------------------------------------
function Kinetica:Slider_Width(wndHandler, wndCtrl, nValue, nOldValue)
	if math.floor(nValue) == math.floor(nOldValue) then return end
	self.wndMain:FindChild("Label_WidthDisplay"):SetText(string.format("%s", math.floor(nValue)))
	self.setWidth = math.floor(nValue)
end


-----------------------------------------------------------------------------------------------
-- Kinetica Instance
-----------------------------------------------------------------------------------------------
local KineticaInst = Kinetica:new()
KineticaInst:Init()

